################################################################################
#
#       This is part of CATBox (Combinatorial Algorithm Toolbox) 
#       version  1.1 from 4/10/2011. You can find more information at 
#       http://schliep.org/CATBox/
#
#	file:   FindpathEuclid.pro
#	author: Torsten Pattberg (torsten.pattberg@klf.de)
#               Alexander Schliep (alexander@schliep.org)
#
#       Copyright (C) 1998-2011, Winfried Hochstaettler and Alexander Schliep.
#	(C) 2010 Springer Verlag.
#       
#	This is part of the Springer textbook "CATBox - an interactive course in 
#       discrete optimization".
#
#       All rights reserved. Do not distribute without written permission. 
#




#
#       This file is version $Revision: 409 $ 
#                       from $Date: 2011-03-23 11:03:38 -0400 (Wed, 23 Mar 2011) $
#             last change by $Author: schliep $.
#
#
################################################################################

#### Options #################################################

breakpoints = [12]
interactive = [1,2]
graphDisplays = 1
about = """<HTML>
<HEAD>
<TITLE>Shortest Path - Find Path Euclid</TITLE>
</HEAD>
<BODY>

This algorithm finds a shortest path from the source to the sink in a graph
with euclidian distances.

</BODY></HTML>
"""

class MyGraphInformer(WeightedGraphInformer):
    def VertexInfo(self,v):
	if dist[v] == 0:
            return "Vertex %d is root of tree, eucl. distance to t is %f" % (v,distance[v])
	elif dist[v] == gInfinity:
            return "Vertex %d is not in tree, eucl. distance to t is %f" % (v,distance[v])
	elif self.G.edgeWeights[0].QInteger():
            return "Vertex %d has predecessor %d and distance %d, eucl. distance to t is %f" % (v,
                                                     pred[v],dist[v],distance[v])
	else:
            return "Vertex %d has predecessor %d and distance %f, eucl. distance to t is %f" % (v,
                                                     pred[v],dist[v],distance[v])

from math import sqrt

def pickCallback(A,v,type):
    if type == "s":
        A.SetVertexAnnotation(v,type,"green")
    else:
        A.SetVertexAnnotation(v,type,"red")
    return

class MyAnimatedVertexPriorityQueue(AnimatedVertexPriorityQueue):
    def DeleteMin(self):
        v = AnimatedVertexPriorityQueue.DeleteMin(self)
        if v and pred[v]:
            A.SetEdgeColor(pred[v], v, 'red')        
        return v

pickCallbackSource = lambda v, type="s", a=A: pickCallback(a,v,type)
PickSource         = lambda f=pickCallbackSource: self.PickVertex(1,None,f)
pickCallbackSink   = lambda v, type="t", a=A: pickCallback(a,v,type)
PickSink           = lambda f=pickCallbackSink: self.PickVertex(G.Order(),None,f)
Neighborhood       = lambda v,a=A,g=G: AnimatedNeighborhood(a,g,v,['#EE9900',"red"])

def ShowPath(u,v):
    w = v
    path = []
    while w is not None:
        path.append(w)
        w = pred[w]
    id = A.HighlightPath(path, "yellow")

length       = G.edgeWeights[0]
pred         = AnimatedPredecessor(A, predColor='#EE9900')
dist         = AnimatedVertexLabeling(A)
PQ           = MyAnimatedVertexPriorityQueue(A, colorOff='red')
distance     = BlinkingVertexLabeling(A)
coo          = G.embedding

for v in G.vertices:
    pred[v] = None
    dist[v] = gInfinity
    distance[v] = gInfinity

A.RegisterGraphInformer(MyGraphInformer(G,"length"))
